#include <qdir.h>
#include <qfile.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <qstring.h>

#include <stdlib.h>

#include <iostream>

#include "brainEnvironment.h"

// OSX specific include to check for bundle location
#if defined(Q_OS_MACX)
#include <CoreFoundation/CoreFoundation.h>
#endif

using namespace std;


brainEnvironment::brainEnvironment( QString environmentFile ) {

  // this is not yet specified in the environment file, so set the
  // options manually

  this->updateRangeOnTextureChange = true;



  // create the different members
  textureDir = new QString();
  textureFile = new QString();
  developmentStatus = 0;
  textureFiles = new QStringList();


  QString defaultEnvironment = "/export/home/claude/dev/Linux-i686/share/brain-view/brain-view-config";

  // on OSX the default Environment file is inside the bundle
#if defined(Q_OS_MACX)
  CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
  CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, 
						kCFURLPOSIXPathStyle);
  const char *pathPtr = CFStringGetCStringPtr(macPath, 
					      CFStringGetSystemEncoding());
  qDebug("Path = %s", pathPtr);
  defaultEnvironment = pathPtr;
  defaultEnvironment.append("/Contents/Resources/brain-view-config");
  CFRelease(pluginRef);
  CFRelease(macPath);

#endif



  // check to see if the file was specified - if not, use one from home dir
  if (environmentFile == "NULL") {
    char *homedir = getenv("HOME");
    environmentFile = homedir;
    environmentFile.append("/.brain-view");

    // if one from home dir doesn't exist, look for the one in the package dir
    if(!QFile::exists(environmentFile)) {
      environmentFile = defaultEnvironment;
    }
  }

  // open the file with the environment settings
  QFile ef( environmentFile );
  if(!ef.open( IO_ReadOnly )) {
    cerr << "WARNING: environment file (~/.brain-view or " << defaultEnvironment << ") could not be found so textures will not be available" << endl;
  } else {
    cout << "Using environment file: " << environmentFile << endl;
    QTextStream ts( &ef );
    
    // search for lines of interest
    while (! ts.atEnd() ) {
      QString line = ts.readLine();
    
      // # == comment; ignore
      if (line.find( QRegExp("^#"), 0 ) != -1) {
        // do nothing
      }
      // ignore blank lines
      else if (line.isEmpty()) {
        // do nothing
      }
      // Texturedir: a directory containing one or more images that can
      // be used as textures
      else if ( line.find( "Texturedir:" ) != -1 ) {
        line = parseLine(line);
        // just to make sure, also append a "/"
        *textureDir = line.append( "/" );
      
        //      for (QStringList::Iterator it = this->textures.begin(); 
        //           it != this->textures.end(); it++) {
        //        cout << *it << endl;
        //      }
      }
      // Texture: the texture file to use; can be either a full path or
      // a file that lives in the textureDir
      else if ( line.find( "Texture:" ) != -1 ) {
        line = parseLine(line);
        *textureFile = line;
      }
      // Development: show the menu items for some things that are
      // still quite broken. The goal is to allow the same code-base
      // to contain items that do not yet work.
      else if ( line.find( "Development:" ) != -1 ) {
	line = parseLine(line); // throwaway line
	developmentStatus = 1;
      }

      // If nothing else is matched print Error
      else {
        cerr << "WARNING: line \"" << line << "\" was not recognised" << endl
             << "while parsing the initialisation file." << endl;
      }
    }

#if defined(Q_OS_MACX)
    if (textureDir->isEmpty()) {
      CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
      CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, 
						    kCFURLPOSIXPathStyle);
      const char *pathPtr = CFStringGetCStringPtr(macPath, 
						  CFStringGetSystemEncoding());
      *textureDir = pathPtr;
      textureDir->append("/Contents/Resources/textures/");
      CFRelease(pluginRef);
      CFRelease(macPath);
    }
#endif

    // test whether the texture file, if given, is an absolute path. If
    // not, append the textureDir
    if ( textureFile->isEmpty() == false && 
         textureFile->find(QRegExp("^/")) == -1 ){
      if ( textureDir->isEmpty() == false ) {
        createTextureList();
        *textureFile = textureFile->prepend( *textureDir );
      }
      else {
        cerr << "WARNING: a texture file was specified in the" << endl
             << "initialisation file, but that file did not have an" << endl
             << "absolute path and a Texturedir variable was not specified!" 
             << endl;
      }
    }

    // print out a summary of what was loaded
    cout << "Initialisation variables:" << endl
         << "   Texturedir: " << *textureDir << endl
         << "   Texturefile: " << *textureFile << endl;
  }
}

/*
 * Takes the string textureDir and opens it as a directory, setting
 * filling the textures list with the entries of that directory. Does
 * not return anything, but prints a warning if no textures were
 * found.
 */
void brainEnvironment::createTextureList() {
  QDir td( *textureDir );
  if ( td.isReadable()) {
    cerr << "inside td.isReadable" << endl;
    td.setFilter( QDir::Files );
    *textureFiles = td.entryList();
  }
  else {
    cerr << "WARNING: could not read texture files in directory " 
         << *textureDir <<  endl;
  }
}

/* Takes a line from the initialisation file and returns the juicy
   bit; i.e. the part that comes after the descriptor. Lines should
   have the following format: Descriptor: item. Returns the so
   modified string */
QString brainEnvironment::parseLine(QString line) {
  line.remove(0, line.find( ":" )+1);
  line = line.stripWhiteSpace();
  return line;
}
  
   
// test code
/*
g++ -o test brainEnvironment.cc -I/usr/local/qt3/include -L/usr/local/qt3/lib -L/usr/X11R6/lib -lqt -lqui -lGL -lSM -lICE -lX11 -lXft -lXinerama
*/
/*
  #ifdef DEBUG
  int main( int argc, char *argv[] ) {
  brainEnvironment be( "/home/bic/jason/.brain-view" );
  return 0;
  #endif

}
*/
